==> phpcore-7.1.0.tgz: package/src/Control/FutureFactory.js <==
/*
 * PHPCore - PHP environment runtime components
 * Copyright (c) Dan Phillimore (asmblah)
 * https://github.com/uniter/phpcore/
 *
 * Released under the MIT license
 * https://github.com/uniter/phpcore/raw/master/MIT-LICENSE.txt
 */

'use strict';

var _ = require('microdash'),
    phpCommon = require('phpcommon'),
    queueMicrotask = require('core-js-pure/actual/queue-microtask'),
    Exception = phpCommon.Exception,
    Pause = require('./Pause');

/**
 * @param {ControlFactory} controlFactory
 * @param {PauseFactory} pauseFactory
 * @param {ValueFactory} valueFactory
 * @param {ControlBridge} controlBridge
 * @param {class} Future
 * @constructor
 */
function FutureFactory(
    controlFactory,
    pauseFactory,
    valueFactory,
    controlBridge,
    Future
) {
    /**
     * @type {ControlBridge}
     */
    this.controlBridge = controlBridge;
    /**
     * @type {ControlFactory}
     */
    this.controlFactory = controlFactory;
    /**
     * @type {class}
     */
    this.Future = Future;
    /**
     * @type {PauseFactory}
     */
    this.pauseFactory = pauseFactory;
    /**
     * @type {ValueFactory}
     */
    this.valueFactory = valueFactory;
}

_.extend(FutureFactory.prototype, {
    /**
     * Creates a new Future to be resolved with the given value after deferring.
     *
     * @param {*} value
     * @returns {Future}
     */
    createAsyncPresent: function (value) {
        return this.createFuture(function (resolve) {
            queueMicrotask(function () {
                resolve(value);
            });
        });
    },

    /**
     * Creates a new Future to be rejected with the given error after deferring.
     *
     * @param {Error} error
     * @returns {Future}
     */
    createAsyncRejection: function (error) {
        return this.createFuture(function (resolve, reject) {
            queueMicrotask(function () {
                reject(error);
            });
        });
    },

    /**
     * Creates a new Future
     *
     * @param {Function} executor
     * @returns {Future}
     */
    createFuture: function (executor) {
        var factory = this,
            // savedCallStack = factory.callStack.save(),
            sequence = factory.controlFactory.createSequence(),
            reject = function reject(error) {
                // factory.callStack.restore(savedCallStack);

                sequence.throwInto(error);
            },
            resolve = function resolve(result) {
                // factory.callStack.restore(savedCallStack);

                sequence.resume(result);
            };

        try {
            executor(resolve, reject);
        } catch (error) {
            if (error instanceof Pause) {
                throw new Exception('Unexpected Pause raised by Future executor');
            }

            // Any errors raised during evaluation of the Future executor should reject the Future
            reject(error);
        }

        return new factory.Future(factory, factory.pauseFactory, factory.valueFactory, sequence);
    },

    /**
     * Creates a new present Future for the given value
     *
     * TODO: Reinstate an actual lightweight Present class to avoid creating Sequences when unnecessary
     *
     * @param {*} value
     * @returns {Future}
     */
    createPresent: function (value) {
        return this.createFuture(function (resolve) {
            resolve(value);
        });
    },

    /**
     * Creates a new rejected Future for the given error
     *
     * @param {Error} error
     * @returns {Future}
     */
    createRejection: function (error) {
        return this.createFuture(function (resolve, reject) {
            reject(error);
        });
    },

    /**
     * Derives a new Future from an existing Sequence
     *
     * @param {Sequence} sequence
     * @returns {Future}
     */
    deriveFuture: function (sequence) {
        var factory = this,
            derivedSequence = factory.controlFactory.createSequence();

        sequence.next(
            function (result) {
                return derivedSequence.resume(result);
            },
            function (error) {
                return derivedSequence.throwInto(error);
            }
        );

        return new factory.Future(factory, factory.pauseFactory, factory.valueFactory, derivedSequence);
    }
});

module.exports = FutureFactory;

==> phpcore-7.2.0.tgz: package/src/Control/FutureFactory.js <==
/*
 * PHPCore - PHP environment runtime components
 * Copyright (c) Dan Phillimore (asmblah)
 * https://github.com/uniter/phpcore/
 *
 * Released under the MIT license
 * https://github.com/uniter/phpcore/raw/master/MIT-LICENSE.txt
 */

'use strict';

var _ = require('microdash'),
    queueMicrotask = require('core-js-pure/actual/queue-microtask');

/**
 * @param {PauseFactory} pauseFactory
 * @param {ValueFactory} valueFactory
 * @param {ControlBridge} controlBridge
 * @param {class} Future
 * @constructor
 */
function FutureFactory(
    pauseFactory,
    valueFactory,
    controlBridge,
    Future
) {
    /**
     * @type {ControlBridge}
     */
    this.controlBridge = controlBridge;
    /**
     * @type {class}
     */
    this.Future = Future;
    /**
     * @type {PauseFactory}
     */
    this.pauseFactory = pauseFactory;
    /**
     * @type {ValueFactory}
     */
    this.valueFactory = valueFactory;
}

_.extend(FutureFactory.prototype, {
    /**
     * Creates a new Future to be resolved with the given value after deferring.
     *
     * @param {*} value
     * @returns {Future}
     */
    createAsyncPresent: function (value) {
        return this.createFuture(function (resolve) {
            queueMicrotask(function () {
                resolve(value);
            });
        });
    },

    /**
     * Creates a new Future to be rejected with the given error after deferring.
     *
     * @param {Error} error
     * @returns {Future}
     */
    createAsyncRejection: function (error) {
        return this.createFuture(function (resolve, reject) {
            queueMicrotask(function () {
                reject(error);
            });
        });
    },

    /**
     * Creates a new Future
     *
     * @param {Function} executor
     * @param {Future=} parent
     * @returns {Future}
     */
    createFuture: function (executor, parent) {
        var factory = this;

        return new factory.Future(
            factory,
            factory.pauseFactory,
            factory.valueFactory,
            factory.controlBridge,
            executor,
            parent || null
        );
    },

    /**
     * Creates a new present Future for the given value
     *
     * TODO: Reinstate an actual lightweight Present class to avoid the complexity of Futures when unnecessary,
     *       however now that Sequence is defunct, perhaps less of an issue?
     *
     * @param {*} value
     * @returns {Future}
     */
    createPresent: function (value) {
        return this.createFuture(function (resolve) {
            resolve(value);
        });
    },

    /**
     * Creates a new rejected Future for the given error
     *
     * @param {Error} error
     * @returns {Future}
     */
    createRejection: function (error) {
        return this.createFuture(function (resolve, reject) {
            reject(error);
        });
    }
});

module.exports = FutureFactory;
